#include "prng.h"
#include <random>
#include "stdint.h"

PRNG::PRNG()
{
    // Init Mersenne Twister  (just included as a reference as it is the standard in most languages)
    mersenneTwister = new std::mt19937;

    // Init Xoroshiro 128+
    s[0] = 0xbeac0467eba5facbULL;
    s[1] = 0xd86b048b86aa9922ULL;

    // Init JKISS32
    _x = 123456789;
    _y = 234567891;
    _z = 345678912;
    _w = 456789123;
    _c = 0;

    // Init PCG   (with the seed Mellissa recommended)
    state = 0x853c49e6748fea9bULL;
    inc = 0xda3e39cb94b95bdbULL;

    // Init P2 LFSR
}

PRNG::~PRNG()
{
    delete mersenneTwister;
}

// Some needed definitions
static inline uint64_t rotl(const uint64_t x, int k) { return (x << k) | (x >> (64 - k)); }



// Get methods for the different PRNG's
uint32_t PRNG::getMersenne()
{
    return distrubution(*mersenneTwister);
}

uint32_t PRNG::getXoroshiro()
{
    const uint64_t s0 = s[0];
    uint64_t s1 = s[1];
    const uint64_t result = s0 + s1;

    s1 ^= s0;
    s[0] = rotl(s0, 55) ^ s1 ^ (s1 << 14); // a, b
    s[1] = rotl(s1, 36); // c

    return static_cast<uint32_t>(result);
}

uint32_t PRNG::getJkiss32()
{
    int32_t t;
    int32_t y = _y;

    y ^= (y << 5);
    y ^= (y >> 7);
    y ^= (y << 22);
    _y = y;
    t = _z + _w + _c;
    _z = _w;
    _c = t < 0;
    _w = t & 0x7FFFFFFF;
    _x += 0x542023AB;

    return (_x + _y + _w);
}

uint32_t PRNG::getPcg()
{
    uint64_t oldstate = state;
    state = oldstate * 6364136223846793005ULL + inc;
    uint32_t xorshifted = ((oldstate >> 18u) ^ oldstate) >> 27u;
    uint32_t rot = oldstate >> 59u;

    return (xorshifted >> rot) | (xorshifted << ((-rot) & 31));
}

uint32_t PRNG::getP2Lfsr()
{
    return 0xBAD;
}
